#!/bin/sh
# Copyright (C) 2013 windriver

usb_device_exist() {
	local vendorID="$1"
	local prodID="$2"

	which lsusb >/dev/null 2>&1 || return 1
	lsusb | grep -q "${vendorID}:${prodID}"  && return 0 || return 1
}

get_modem_sysdir_byName() {
	# $1: manufacturer name
	# $2: product name 
	local sysdirroot="/sys/class/usb_device"
	local manuName=""
	local prodName=""

	for devdir in $(ls ${sysdirroot}) ; do
		manuName=`cat $sysdirroot/$devdir/device/manufacturer 2>/dev/null`
		prodName=`cat $sysdirroot/$devdir/device/product 2>/dev/null`
		[ "$1" == "$manuName" -a "$2" == "$prodName" ] && {
			echo "$sysdirroot/$devdir/device"
			break
		}
	done
}

scan_3g_modem() {
	local pppddev=""
	local manuName=""
	local prodName=""
	local vendorID=""
	local prodID=""
	local sn=""
	local rev=""
	local conntype=""
	local pppddev_all=""
	local modem_keywords=""
	local wwan_proto=""
	local idx=0
	local found_modem=0
	echo "into scan_3g_modem"
	# Stop pppd/sierra connectionmgr to use atcommand
	pkill pppd
	ps -ef | grep "connectionmgr" | grep -v "grep" >/dev/null 2>&1 && {
		/etc/init.d/swconnman stop >/dev/null 2>&1
	}

	# Get modem information which is supported default
	[ -e /etc/modem_cell_default ] && {
		while read modem_rule
		do
			echo "$modem_rule" | grep "#" >/dev/null 2>&1 && continue

			vendorID=`echo "$modem_rule" | awk -F'|' '{print $3}'`
			prodID=`echo "$modem_rule" | awk -F'|' '{print $4}'`
			echo "VID: $vendorID , PID: $prodID"
			usb_device_exist "$vendorID" "$prodID" && found_modem=1
			echo "found_modem : $found_modem"
			[ $found_modem -eq 1 ] && {
				manuName=`echo "$modem_rule" | awk -F'|' '{print $1}'`
				prodName=`echo "$modem_rule" | awk -F'|' '{print $2}'`
				conntype=`echo "$modem_rule" | awk -F'|' '{print $5}'`
				pppddev=`echo "$modem_rule" | awk -F'|' '{print $6}'`
				statedev=`echo "$modem_rule" | awk -F'|' '{print $7}'`
				break
			}
		done < /etc/modem_cell_default 
	}

	# Update old modem information if cannot find modem
	[ $found_modem -eq 0 -o -z "$pppddev"  -o -z "$statedev" ] && {
		uci get network.modem_cell >/dev/null 2>&1 && {
			uci -q set network.modem_cell.present="No"
			uci -q delete network.modem_cell.protoall
			uci -q delete network.modem_cell.pppddev
			uci -q delete network.modem_cell.statedev
			uci -q delete network.modem_cell.Vendor
			uci -q delete network.modem_cell.ProdID
			uci -q delete network.modem_cell.Manufacturer
			uci -q delete network.modem_cell.Product
			uci -q delete network.modem_cell.SerialNumber
			uci -q delete network.modem_cell.Rev
			echo "set the proto as none"
			uci -q set network.wwan.proto="none"
			uci commit network
			rm -rf /tmp/.uci/network
		}
		return 1
	}

	# Get other information by atcommand	
	which atcommand >/dev/null 2>&1 && [ -n "$statedev" ] && {
		sn=`atcommand $statedev sn`
		rev=`atcommand $statedev revision`
	}

	# Save modem info
	uci get network.modem_cell >/dev/null 2>&1 || {
		touch /etc/config/network
		uci set network.modem_cell="device"
	}
	uci set network.modem_cell.present="Yes"
	uci set network.modem_cell.protoall="$conntype"
	uci set network.modem_cell.pppddev="/dev/$pppddev"
	uci set network.modem_cell.statedev="/dev/$statedev"
	uci set network.modem_cell.Manufacturer="$manuName"
	uci set network.modem_cell.Product="$prodName"
	uci set network.modem_cell.Vendor="$vendorID"
	uci set network.modem_cell.ProdID="$prodID"
	uci set network.modem_cell.SerialNumber="$sn"
	uci set network.modem_cell.Rev="$rev"

	echo "before update wwan interface"
	# Update wwan interface
	uci get network.wwan >/dev/null 2>&1 && {
		# Check wwan proto (proto may be changed to 'none' to disable interface)
		wwan_proto=`uci get network.wwan.proto`
		echo "conntype $conntype"
		if [ $conntype != 'none' ]; then
			echo 'conntype is not none'
			uci set network.wwan.proto="$conntype"
		fi
		conntype="$conntype none"
		echo "wwan_proto : $wwan_proto , conntype : $conntype ."
		
		echo "$conntype" | grep "$wwan_proto" >/dev/null 2>&1 || {
			echo "into uci set network.wwan.proto"
			wwan_init_proto=`echo $conntype | awk '{print $1}'`
			wwan_init_proto=${wwan_init_proto:-$wwan_proto}
			echo "wwan_init_proto $wwan_init_proto"
			uci set network.wwan.proto="$wwan_init_proto"
		}

		# Set wwan ifname
		wwan_proto=`uci get network.wwan.proto`
		case "$wwan_proto" in
		3g) 	uci set network.wwan.ifname="3g-wwan" ;;
		sconn)	uci set network.wwan.ifname="usb0" ;;
		esac

		#set wwan device
		uci set network.wwan.device="/dev/$pppddev"

		#set wwan service
		case "$prodName" in
		MC8355) uci set network.wwan.sconnservice="UMTS" ;;
		MC7750) uci set network.wwan.sconnservice="LTE" ;;
		DE910) uci set network.wwan.service="evdo" ;;
		esac
	}

	# Commit all the changes	
	uci commit network
	rm -rf /tmp/.uci/network	
	return 0
}

scan_sim_card() {
	local imsi=""
	local operator_code=""
	local operator=""
	local apn=""
	local country=""
	local service=""
	local sconnservice=""
	local sim_information=""
	local dev=""
	local old_operator_code=`uci -q get network.sim_card.operator_code`
	local old_operator=`uci -q get network.sim_card.operator`

	clear_sim_card_info() {
		uci get network.sim_card >/dev/null 2>&1 && {
			uci set network.sim_card.present="Unknown"
			uci -q delete network.sim_card.IMSI
			uci -q delete network.sim_card.operator
			uci -q delete network.sim_card.operator_code
			uci commit network
			rm -f /tmp/.uci/network
		}
	}

	# Stop pppd/sierra connectionmgr to use atcommand
	pkill pppd
	ps -ef | grep "connectionmgr" | grep -v "grep" >/dev/null 2>&1 && {
		/etc/init.d/swconnman stop >/dev/null 2>&1
	}

	# Return when modem device in not found.
	dev=`uci -q get network.modem_cell.statedev`
	[ -z "$dev" ] && {
		clear_sim_card_info
		return 2
	}

	# Network operator Name in different modems are not the same. 
	# So it's better to use the code number of operator.
	which atcommand >/dev/null 2>&1 && {
		imsi=`atcommand $statedev imsi`
		atcommand $statedev -cmd AT+COPS=3,2 >/dev/null 2>&1
		operator_code=`atcommand $statedev -cmd AT+COPS? | awk -F"\"" '{print $2}'`
		atcommand $statedev -cmd AT+COPS=3,0 >/dev/null 2>&1
		operator=`atcommand $statedev -cmd AT+COPS? | awk -F"\"" '{print $2}'`
	}

	# No SIM card detected, then return without changing any other options.
	[ -z "$imsi" -a -z "$operator" -a -z "$operator_code" ]  && {
		clear_sim_card_info
		return 1
	}

	uci get network.sim_card >/dev/null 2>&1 || {
		touch /etc/config/network
		uci set network.sim_card="device"
	}
	uci set network.sim_card.present="Yes"
	uci set network.sim_card.IMSI="$imsi"
	uci set network.sim_card.operator="$operator"
	uci set network.sim_card.operator_code="$operator_code"
	uci commit network
	rm -f /tmp/.uci/network

	# The SIM card information has been set and is the same this time.

	[ -n "$old_operator_code" -a "$old_operator_code" == "$operator_code" ] && {
		return 0
	}
	[  -n "$old_operator" -a "$old_operator" == "$operator" ] && {
		return 0
	}

	# No wwan interface in /etc/config/network
	uci get network.wwan >/dev/null 2>&1 || return 0

	# Set wwan interface according to SIM card.
	[ -n "$imsi" ] && {
		MNC_2=`echo ${imsi:0:5}`
		MNC_3=`echo ${imsi:0:6}`
		sim_information=`grep ":$MNC_2:" /etc/sim_card_operator_default 2>/dev/null`
		[ -z "$sim_information" ] && {
			sim_information=`grep ":$MNC_3:" /etc/sim_card_operator_default 2>/dev/null`
		}
	}
	[ -n "$operator_code" -a -z "$sim_information" ] && {
		sim_information=`grep ":$operator_code:" /etc/sim_card_operator_default 2>/dev/null`
	}
	[ -n "$operator" -a -z "$sim_information" ] && {
		sim_information=`grep ":$operator:" /etc/sim_card_operator_default 2>/dev/null`
	}
	[ -n "$sim_information" ] && {
		country=`echo $sim_information | awk -F":" '{print $1}'`
		apn=`echo $sim_information | awk -F":" '{print $4}'`
		service=`echo $sim_information | awk -F":" '{print $7}'`
		sconnservice=`echo $sim_information | awk -F":" '{print $8}'`
		uci set network.wwan.country="$country"
		uci set network.wwan.apn="$apn"
		[ -n "$service" ] && uci set network.wwan.service="$service"
		[ -n "$sconnservice" ] && uci set network.wwan.sconnservice="$sconnservice"
		uci commit network
		rm -f /tmp/.uci/network
	}	

	return 0
}	

scan_zigbee_modem() {
	local manuName prodName devName vendorID prodID
	local VersionInfo btversion macversion
	local found_modem=0

	# Get modem information which is supported default
	[ -e /etc/modem_zigbee_default ] && {
		while read modem_rule
		do
			echo "$modem_rule" | grep "#" >/dev/null 2>&1 && continue

			vendorID=`echo "$modem_rule" | awk -F'|' '{print $3}'`
			prodID=`echo "$modem_rule" | awk -F'|' '{print $4}'`
			usb_device_exist $vendorID $prodID && found_modem=1
			[ $found_modem -eq 1 ] && {
				manuName=`echo "$modem_rule" | awk -F'|' '{print $1}'`
				prodName=`echo "$modem_rule" | awk -F'|' '{print $2}'`
				devName=`echo "$modem_rule" | awk -F'|' '{print $5}'`
				break
			}
		done < /etc/modem_zigbee_default 
	}
	
	# Cannot find the default modems
	[ $found_modem -eq 0 ] && {
		uci get zigbee.info >/dev/null 2>&1 && {
			uci set zigbee.info.present="No"
			uci -q delete zigbee.info.Vendor
			uci -q delete zigbee.info.ProdID
			uci -q delete zigbee.info.Manufacturer
			uci -q delete zigbee.info.Product
			uci -q delete zigbee.info.Device
			uci -q delete zigbee.info.BootloaderVer
			uci -q delete zigbee.info.MACVer
			uci commit zigbee
			rm -rf /tmp/.uci/zigbee
		}
		return 1
	}

	# Update modem info
	uci get zigbee.info >/dev/null 2>&1 || {
		touch /etc/config/zigbee
		uci set zigbee.info="device"
	}
	uci set zigbee.info.present="Yes"
	uci set zigbee.info.Vendor="$vendorID"
	uci set zigbee.info.ProdID="$prodID"
	uci set zigbee.info.Manufacturer="$manuName"
	uci set zigbee.info.Product="$prodName"
	uci set zigbee.info.Device="$devName"
	uci -q delete zigbee.info.BootloaderVer
	uci -q delete zigbee.info.MACVer

	[ "$prodName" == "TI CC2531 USB CDC" -a -n "$devName" ] && {
		fuser $devName >/dev/null 2>&1 || {
			FRI_MAC_Reset $devName >/dev/null 2>&1
			VersionInfo=`FRI_BootLoader_GetVersion 2>/dev/null`
			[ -n "$VersionInfo" ] && {
				btversion=`echo "$VersionInfo" | grep "Bootloader" | tr -d '\r'`
				macversion=`echo "$VersionInfo" | grep "MAC Firmware" | tr -d '\r'`
				uci set zigbee.info.BootloaderVer="$btversion"
				uci set zigbee.info.MACVer="$macversion"
			}
		}	
	}

	uci commit zigbee
	rm -f /tmp/.uci/zigbee
	return 0
}
